@@ -68,12 +68,13 @@ void check1dir(char *dir)
 
     if (stat(dir, &sbuf)) {
 	if (errno != ENOENT || mkdir(dir, set_dirmode) < 0) {
-	    sprintf(errmsg, "%s \"%s\".",
-		    lang[MSG_CANNOT_CREATE_DIRECTORY], dir);
-	    progerr(errmsg);
-	    return;
+            if (errno != EEXIST) {
+	        sprintf(errmsg, "%s \"%s\".",
+			lang[MSG_CANNOT_CREATE_DIRECTORY], dir);
+		progerr(errmsg);
+	    }
 	}
-	else if (set_showprogress)
+	if (set_showprogress)
 	    printf(" %s \"%s\", %s %o.\n",
 		   lang[MSG_CREATING_DIRECTORY], dir,
 		   lang[MSG_MODE], set_dirmode);
@@ -107,12 +108,13 @@ void checkdir(char *dir)
 	    *p = '\0';
 	    if (stat(dir, &sbuf)) {
 		if (errno != ENOENT || mkdir(dir, set_dirmode) < 0) {
-		    sprintf(errmsg, "%s \"%s\".",
-			    lang[MSG_CANNOT_CREATE_DIRECTORY], dir);
-		    progerr(errmsg);
-		    return;
+		    if (errno != EEXIST) {
+		        sprintf(errmsg, "%s \"%s\".",
+				lang[MSG_CANNOT_CREATE_DIRECTORY],dir);
+			progerr(errmsg);
+		    }
 		}
-		else if (set_showprogress)
+		if (set_showprogress)
 		    printf(" %s \"%s\", %s %o.\n",
 			   lang[MSG_CREATING_DIRECTORY], dir,
 			   lang[MSG_MODE], set_dirmode);
@@ -359,7 +361,6 @@ int find_max_msgnum()
     }
     closedir(dir);
     free(s_dir);
-    max_msgnum = max_num;
     return max_num;
 }
 
@@ -475,16 +476,31 @@ struct emailsubdir *msg_subdir(int msgnum, time_t date)
  */
 
 char *msg_href(struct emailinfo *to_email, struct emailinfo *from_email)
+/* note: you probably have to make a copy of
+ * the buffer returned before the next call to this function.
+ */
+{
+    static char buffer[MAXFILELEN+11];
+    trio_snprintf(buffer,MAXFILELEN+11, "<a href=\"%s\">",
+		    msg_relpath(to_email, from_email));
+    return buffer;
+}
+
+
+char *msg_relpath(struct emailinfo *to_email, struct emailinfo *from_email)
+/* called by msg_href() : note: you probably have to make a copy of
+ * the buffer returned before the next call to this function.
+ */
 {
     static char buffer[MAXFILELEN];
     if (!from_email && to_email->subdir)
-        trio_snprintf(buffer, MAXFILELEN, "<a href=\"%s%.4d.%s\">",
+        trio_snprintf(buffer, MAXFILELEN, "%s%.4d.%s",
 		  to_email->subdir->subdir, to_email->msgnum, set_htmlsuffix);
     else if(!to_email->subdir || to_email->subdir == from_email->subdir)
-        trio_snprintf(buffer, MAXFILELEN, "<a href=\"%.4d.%s\">",
+        trio_snprintf(buffer, MAXFILELEN, "%.4d.%s",
 		  to_email->msgnum, set_htmlsuffix);
     else
-        trio_snprintf(buffer, MAXFILELEN, "<a href=\"%s%s%.4d.%s\">",
+        trio_snprintf(buffer, MAXFILELEN, "%s%s%.4d.%s",
 		  to_email->subdir->rel_path_to_top,
 		  to_email->subdir->subdir, to_email->msgnum, set_htmlsuffix);
     return buffer;
@@ -508,6 +524,17 @@ char *htmlfilename(const char *file, struct emailinfo *email,
     return buf;
 }
 
+char *haofname(struct emailinfo *email)
+{
+    char *buf;
+	trio_asprintf(&buf,"%s%s", 
+		email && email->subdir ? email->subdir->full_path : set_dir, 
+		HAOF_NAME);
+    return buf;
+
+}
+
+
 /* matches_existing returns 0 if it finds a file with the same msgnum as
  argument eptr but different contents. A return value of 1 does not
  guarantee that they match, it only says a difference wasn't found. */
