@@ -130,7 +130,11 @@ void usage(void)
     printf("  -b URL        : %s\n", lang[MSG_OPTION_B]);
     printf("  -c file       : %s\n", lang[MSG_OPTION_C]);
     printf("  -d dir        : %s\n", lang[MSG_OPTION_D]);
+#ifdef GDBM
     printf("  -g            : %s\n", lang[MSG_OPTION_G]);
+#else
+    printf("  [ -g            : %s ]\n", lang[MSG_OPTION_G_NOT_BUILD_IN]);
+#endif
     printf("  -i            : %s\n", lang[MSG_OPTION_I]);
     printf("  -l label      : %s\n", lang[MSG_OPTION_L]);
     printf("  -m mbox       : %s\n", lang[MSG_OPTION_M]);
@@ -145,6 +149,7 @@ void usage(void)
     printf("  -v            : %s\n", lang[MSG_OPTION_VERBOSE]);
     printf("  -V            : %s\n", lang[MSG_OPTION_VERSION]);
     printf("  -x            : %s\n", lang[MSG_OPTION_X]);
+    printf("  -X            : %s\n", lang[MSG_OPTION_XML]);
     printf("  -1            : %s\n", lang[MSG_OPTION_1]);
     printf("  -L lang       : %s (", lang[MSG_OPTION_LANG]);
 
@@ -189,14 +194,10 @@ int main(int argc, char **argv)
 
     opterr = 0;
 
+#define GETOPT_OPTSTRING ("a:Ab:c:d:gil:L:m:n:o:ps:tTuvVxX0:1M?")
+
     /* get pre config options here */
-    while ((i = getopt(argc,argv,
-#ifdef GDBM
-			    "a:Ab:c:d:gil:L:m:n:o:ps:tTuvVx0:1M?"
-#else
-			    "a:Ab:c:d:il:L:m:n:o:ps:tTuvVx0:1M?"
-#endif
-			    )) != -1) {
+    while ((i = getopt(argc,argv, GETOPT_OPTSTRING )) != -1) {
         switch((char) i) {
 	case 'c':
 	    configfile = strreplace(configfile, optarg);
@@ -206,13 +207,12 @@ int main(int argc, char **argv)
 	    break;
 	case 'V':
 	    version();
-	 /*NOTREACHED*/ case 'a':
+	 /*NOTREACHED*/ 
+	case 'a':
 	case 'A':
 	case 'b':
 	case 'd':
-#ifdef GDBM
 	case 'g':
-#endif
 	case 'i':
 	case 'l':
 	case 'L':
@@ -225,6 +225,7 @@ int main(int argc, char **argv)
 	case 'T':
 	case 'u':
 	case 'x':
+	case 'X':
 	case '0':
 	case '1':
 	case 'M':
@@ -252,13 +253,7 @@ int main(int argc, char **argv)
 
     /* now get the post-config options! */
 
-    while ((i = getopt(argc,argv,
-#ifdef GDBM
-			    "a:Ab:c:d:gil:L:m:n:o:ps:tTuvx0:1M?"
-#else
-			    "a:Ab:c:d:il:L:m:n:o:ps:tTuvx0:1M?"
-#endif
-      )) != -1) {
+    while ((i = getopt(argc,argv, GETOPT_OPTSTRING)) != -1) {
         switch((char) i) {
 	case 'A':
 	    set_append = 1;
@@ -275,11 +270,9 @@ int main(int argc, char **argv)
 	case 'd':
 	    set_dir = strreplace(set_dir, optarg);
 	    break;
-#ifdef GDBM
 	case 'g':
 	    set_usegdbm = 1;
 	    break;
-#endif
 	case 'i':
 	    use_stdin = TRUE;
 	    break;
@@ -319,6 +312,9 @@ int main(int argc, char **argv)
 	case 'x':
 	    set_overwrite = TRUE;
 	    break;
+	case 'X':
+	    set_writehaof = TRUE;
+	    break;
 	case '0':
 	    set_delete_msgnum = add_list(set_delete_msgnum, optarg);
 	    break;
@@ -376,6 +372,13 @@ int main(int argc, char **argv)
     if (print_usage)		/* Print the usage message and terminate */
 	usage();
 
+#ifndef GDBM
+    if (set_usegdbm) {
+    fprintf(stderr, "%s: %s\n", PROGNAME, lang[MSG_OPTION_G_NOT_BUILD_IN]);
+    usage();
+    }
+#endif
+
     if (set_mbox && !strcasecmp(set_mbox, "NONE")) {
 	use_stdin = TRUE;
     }
@@ -577,7 +580,7 @@ int main(int argc, char **argv)
 
 	if (set_show_msg_links) {
 	    fixnextheader(set_dir, amount_old, -1);
-	    for (i = amount_old; i < amount_new; ++i) {
+	    for (i = amount_old; i <= max_msgnum; ++i) {
 		if (set_showreplies)
 		    fixreplyheader(set_dir, i, 0, amount_old);
 		fixthreadheader(set_dir, i, amount_old);
@@ -601,8 +604,27 @@ int main(int argc, char **argv)
     if (amount_new) {		/* Always write the index files */
 	if (set_linkquotes) {
 	    threadlist = NULL;
+	    threadlist_end = NULL;
 	    printedthreadlist = NULL;
+	    for (i = 0; i <= max_msgnum; ++i) {
+	        struct emailinfo *ep;
+		if (hashnumlookup(i, &ep)) {
+		    ep->flags &= ~USED_THREAD;
+#ifdef FASTREPLYCODE
+		    ep->isreply = 0;
+#endif
+		}
+		threadlist_by_msgnum[i] = NULL;
+	    } /* redo threading with more complete info than in 1st pass */
 	    crossindexthread1(datelist);
+	    for (i = 0; i <= max_msgnum; ++i) {
+	        struct emailinfo *ep, *etmp;
+		hashnumlookup(i, &ep);
+		etmp = nextinthread(i);
+		if (etmp && ep->initial_next_in_thread != etmp->msgnum)
+		    fixthreadheader(set_dir, etmp->msgnum, amount_new);
+		/* if (ep->flags & THREADING_ALTERED) */
+	    }
 	}
 	count_deleted(max_msgnum + 1);
 	if (show_index[0][DATE_INDEX])
@@ -616,6 +638,8 @@ int main(int argc, char **argv)
 	if (set_attachmentsindex) {
 	    writeattachments(amount_new, NULL);
 	}
+	if (set_writehaof) 
+		writehaof(amount_new,NULL);
 	if (set_folder_by_date || set_msgsperfolder)
 	    write_toplevel_indices(amount_new);
 	if (set_monthly_index || set_yearly_index)
