@@ -1096,6 +1096,8 @@ int parsemail(char *mbox,	/* file name */
     int alternative_weight = -1;	/* the current weight of the prefered alternative content */
     struct body *alternative_lp = NULL;	/* the previous alternative lp */
     struct body *alternative_bp = NULL;	/* the previous alternative bp */
+    struct body *append_bp = NULL; /* text to append to body after parse done*/
+    struct body *append_lp = NULL;
     FileStatus alternative_lastfile_created = NO_FILE;	/* previous alternative attachments, for non-inline MIME types */
     char alternative_file[129];	/* file name where we store the non-inline alternatives */
     char alternative_lastfile[129];	/* last file name where we store the non-inline alternatives */
@@ -1496,11 +1498,8 @@ int parsemail(char *mbox,	/* file name */
 				cp++;	/* pass a quote too if one is there */
 
 			    sscanf(cp, "%128[^;\"\n]", charbuffer);
-			    if (strcasecmp("ISO-8859-1", charbuffer) &&
-				strcasecmp("US-ASCII", charbuffer))
-				/* If this isn't any of the charsets above,
-				   then save the charset info! */
-				charset = strsav(charbuffer);
+			    /* save the charset info */
+			    charset = strsav(charbuffer);
 			}
 
 			if (alternativeparser) {
@@ -1677,7 +1676,8 @@ int parsemail(char *mbox,	/* file name */
 #if DEBUG_PARSE
 					printf("Error, new message found instead of boundary!\n");
 #endif
-					origbp = append_body(origbp, &origlp, bp);
+					if (bp != origbp)
+					  origbp = append_body(origbp, &origlp, bp);
 					bp = origbp;
 					lp = origlp;
 					boundary = NULL;
@@ -1875,6 +1875,11 @@ int parsemail(char *mbox,	/* file name */
 		if (!inreply)
 		    inreply = oneunre(subject);
 
+		if (append_bp && append_bp != bp) {
+		    bp = append_body(bp, &lp, append_bp);
+		    append_bp = append_lp = NULL;
+		}
+
 		while (rmlastlines(bp));
 
 		emp =
@@ -2018,7 +2023,8 @@ int parsemail(char *mbox,	/* file name */
 #if DEBUG_PARSE
 				printf("We DUMP the chosen alternative\n");
 #endif
-				origbp = append_body(origbp, &origlp, bp);
+				if (bp != origbp)
+				    origbp = append_body(origbp, &origlp, bp);
 				bp = origbp;
 				lp = origlp;
 
@@ -2367,14 +2373,14 @@ int parsemail(char *mbox,	/* file name */
 				    }
 
 				    /* Print attachment comment before attachment */
-				    bp =
-					addbody(bp, &lp, buffer,
+				    append_bp =
+					addbody(append_bp, &append_lp, buffer,
 						BODY_HTMLIZED | bodyflags);
 				    trio_snprintf(buffer, sizeof(buffer),
 					     "<!-- attachment=\"%.80s\" -->\n",
 					     file);
-				    bp =
-					addbody(bp, &lp, buffer,
+				    append_bp =
+					addbody(append_bp, &append_lp, buffer,
 						BODY_HTMLIZED | bodyflags);
 				}
 			    }
@@ -2419,6 +2425,11 @@ int parsemail(char *mbox,	/* file name */
 	if (!inreply)
 	    inreply = oneunre(subject);
 
+	if (append_bp && append_bp != bp) {
+	    bp = append_body(bp, &lp, append_bp);
+	    append_bp = append_lp = NULL;
+	}
+
 	while (rmlastlines(bp));
 
 	strcpymax(fromdate, dp ? dp : "", DATESTRLEN);
@@ -2515,8 +2526,7 @@ int parsemail(char *mbox,	/* file name */
     crossindex();
     threadlist = NULL;
     printedthreadlist = NULL;
-    if (!set_linkquotes)	/* postpone until later if set_linkquotes */
-	crossindexthread1(datelist);
+    crossindexthread1(datelist);
 #if DEBUG_THREAD
     {
 	struct reply *r;
@@ -2625,7 +2635,9 @@ int parse_old_html(int num, struct emailinfo *ep, int parse_body,
 		    }
 		}
 		else if (!strcasecmp(command, "id")) {
-		    msgid = getvalue(line);
+		    char *raw_msgid = getvalue(line);
+		    msgid = unspamify(raw_msgid);
+		    if (raw_msgid) free(raw_msgid);
 		    if (msgid && (!isalnum(*msgid) || !strchr(msgid,'.')
 				  || !strstr(line,"-->")) && set_linkquotes)
 		        msgid = NULL;/* old version of Hypermail wrote junk? */
@@ -2801,6 +2813,8 @@ static int loadoldheadersfrommessages(char *dir, int num_from_gdbm)
     struct reply *replylist_tmp = NULL;
     int first_read_body = 0;
 
+    if (max_num > max_msgnum)
+	max_msgnum = max_num;
     if (set_searchbackmsgnum) {
 	first_read_body = max_num - set_searchbackmsgnum;
 	if (first_read_body < 0)
@@ -2914,12 +2928,9 @@ static int loadoldheadersfromGDBMindex(char *dir)
 {
       char *indexname;
       GDBM_FILE gp;
-      struct body *bp = NULL;
-      struct body *lp = NULL;
       int num;
       int num_added = 0;
 
-      bp = addbody(bp, &lp, "\0", 0);
       authorlist = subjectlist = datelist = NULL;
 
       /* Use gdbm performance hack: instead of opening each and
@@ -2974,6 +2985,9 @@ static int loadoldheadersfromGDBMindex(char *dir)
 	  long exp_time = -1;
 	  int is_deleted = 0;
 	  struct emailinfo *emp;
+	  struct body *bp = NULL;
+	  struct body *lp = NULL;
+	  bp = addbody(bp, &lp, "\0", 0);
 
 	  content = gdbm_fetch(gp, key);
 	  if(!(dp = content.dptr)) {
@@ -3019,7 +3033,7 @@ static int loadoldheadersfromGDBMindex(char *dir)
 		  ++num_added;
 	      if (num == max_num) {
 		  char *filename = articlehtmlfilename(emp);
-		  if (!isfile(filename)) {
+		  if (!isfile(filename) && !is_deleted) {
 		      trio_snprintf(errmsg, sizeof(errmsg),
 			       "%s \"%s\". If you deleted files,"
 			       " you need to delete the gdbm file %s as well.",
@@ -3032,6 +3046,11 @@ static int loadoldheadersfromGDBMindex(char *dir)
 	  }
 	  free(subject);
 	  free(inreply);
+	  if(bp) {
+	      if (bp->line) 
+		  free(bp->line);
+	      free(bp);
+	  }
 
 	  if (!(num % 10) && set_showprogress) {
 	    printf("\r%4d", num);
@@ -3073,11 +3092,6 @@ static int loadoldheadersfromGDBMindex(char *dir)
       } /* end case of could not read gdbm index */
 
       free(indexname);
-      if(bp) {
-        if (bp->line) 
-	  free(bp->line);
-        free(bp);
-      }
 
       return num_added;
 
@@ -3251,6 +3265,8 @@ void fixreplyheader(char *dir, int num, int remove_maybes, int max_update)
 	if (subjmatch && remove_maybes)
 	  return;
       }
+      if (replynum == -1)
+	  return;
     }
     else {
 	if (!email->inreplyto || !email->inreplyto[0])
@@ -3396,6 +3412,8 @@ void fixthreadheader(char *dir, int num, int max_update)
 		    fprintf(fp, "%s: \"%s\"</a>\n",
 			    name, ptr = convchars(subject));
 		    free(ptr);
+		    if (bp->next && strstr(bp->next->line, lang[MSG_NEXT_IN_THREAD]))
+		        bp = bp->next; /* skip old copy of this line */
 		}
 	    }
 	    bp = bp->next;
