@@ -5,7 +5,6 @@
 int ignorecontent(char *);
 int inlinecontent(char *);
 int preferedcontent(int *, char *);
-char *tmpname(char *, char *);
 char *safe_filename(char *);
 char *getmaildate(char *);
 char *getfromdate(char *);
@@ -20,8 +19,8 @@ void crossindex(void);
 void crossindexthread1(struct header *);
 void crossindexthread2(int);
 void fixnextheader(char *, int, int);
-void fixreplyheader(char *, int, int);
-void fixthreadheader(char *, int);
+void fixreplyheader(char *, int, int, int);
+void fixthreadheader(char *, int, int);
 int isre(char *, char **);
 char *findre(char *, char **);
 int textcontent(char *);
