@@ -537,15 +537,60 @@ void print_index_footer_links(FILE *fp, mindex_t called_from,
 #endif
 }
 
+/*----------------------------------------------------------------------------*/
+
+void print_haof_indices(FILE *fp, struct emailsubdir *subdir)
+{
+    int dlev = (subdir != NULL);
+
+    fprintf(fp, "    <indices>\n");
+
+    if (show_index[dlev][DATE_INDEX])
+    	fprintf(fp,"       <dateindex>%s</dateindex>\n",
+				index_name[dlev][DATE_INDEX]);
+
+    if (show_index[dlev][AUTHOR_INDEX])
+    	fprintf(fp,"       <subjectindex>%s</subjectindex>\n",
+				index_name[dlev][AUTHOR_INDEX]);
+
+    if (show_index[dlev][THREAD_INDEX])
+        fprintf(fp,"       <threadindex>%s</threadindex>\n",
+				index_name[dlev][THREAD_INDEX]);
+
+    if (show_index[dlev][SUBJECT_INDEX])
+    	fprintf(fp,"       <authorindex>%s</authorindex>\n",
+				index_name[dlev][AUTHOR_INDEX]);
+
+    if (show_index[dlev][ATTACHMENT_INDEX])
+    	fprintf(fp,"       <attachmentindex>%s</attachmentindex>\n",
+				index_name[dlev][ATTACHMENT_INDEX]);
+
+    fprintf(fp, "    </indices>\n\n");
+}
+
 /*
 ** Prints a comment in the file.
 */
 
 void printcomment(FILE *fp, char *label, char *value)
 {
-    if (value && *value)
+    if (value && *value) {
 	/* we only save the non-NULL ones */
-	fprintf(fp, "<!-- %s=\"%s\" -->\n", label, value);
+	char *ext_value = value;
+	char *ptr = strchr(value, '@');
+	if (set_spamprotect_id && ptr && (!strcmp(label, "id")
+				       || !strcmp(label, "inreplyto"))) {
+	    struct Push retbuf;
+	    INIT_PUSH(retbuf);
+	    PushNString(&retbuf, value, ptr - value);
+	    PushNString(&retbuf, "_at_", 4);
+	    PushString(&retbuf, ptr + 1);
+	    ext_value = PUSH_STRING(retbuf);
+	}
+	fprintf(fp, "<!-- %s=\"%s\" -->\n", label, ext_value);
+	if (ext_value != value)
+	    free(ext_value);
+    }
 }
 
 /*
@@ -885,9 +930,14 @@ void printbody(FILE *fp, struct emailinfo *email, int maybe_reply)
 
     int inquote;
     int quote_num;
-    int quoted_percent = (set_quote_hide_threshold <= 100
-			  ? compute_quoted_percent(bp) : 100);
-    bool replace_quoted = (quoted_percent > set_quote_hide_threshold);
+    int quoted_percent;
+    bool replace_quoted;
+    /* if (set_quote_hide_threshold < 100 && !email->is_deleted) */
+    if (set_quote_hide_threshold <= 100)
+	quoted_percent = compute_quoted_percent(bp);
+    else
+	quoted_percent = 100;
+    replace_quoted = (quoted_percent > set_quote_hide_threshold);
 
     if(set_showprogress && replace_quoted)
       printf("\nMessage %d quoted text (%d %%) replaced by links\n",
@@ -1012,7 +1062,14 @@ void printbody(FILE *fp, struct emailinfo *email, int maybe_reply)
 	    }
 
 	    if (!inheader && (bp->line)[0] == '\n')
-		fprintf(fp, "<p>");
+        /* within the <pre></pre> statements you do not need to
+           insert <p> statements since text is already preformated.
+           the W3C HTML validation script fails for such pages 
+                Akis Karnouskos <akis@ceid.upatras.gr>     */
+                {
+                if (!pre)
+                        fprintf(fp, "<p>");
+                }
 	    else {
 		if (insig) {
 		    ConvURLs(fp, bp->line, id, subject);
@@ -1109,6 +1166,24 @@ print_leading_whitespace(FILE *fp, char *sp)
     return sp;
 }
 
+static int
+has_new_replies(struct emailinfo *email)
+{
+    struct reply *rp;
+    static int max_old_msgnum = -2;
+    if (max_old_msgnum == -2)
+	max_old_msgnum = find_max_msgnum();
+    if (email->msgnum == max_old_msgnum)
+	return 1;  /* next msg needs to be linked, even if no new reply */
+#ifdef FASTREPLYCODE
+    for (rp = email->replylist; rp != NULL; rp = rp->next) {
+	if (rp->frommsgnum == email->msgnum && rp->msgnum > max_old_msgnum)
+	    return 1;
+    }
+#endif
+    return 0;
+}
+
 /*
  * Perform deletions on old messages when run in incremental mode.
  */
@@ -1126,8 +1201,12 @@ void update_deletions(int num_old)
 	    continue;		/* new message - already done */
 	if (hashnumlookup(num, &ep)) {
 	    char *filename = articlehtmlfilename(ep);
-	    if (set_delete_level != DELETE_REMOVES_FILES)
-	        writearticles(num, num + 1);
+	    if (set_delete_level != DELETE_REMOVES_FILES) {
+		struct body *bp = ep->bodylist;
+		if (bp == NULL)
+		    parse_old_html(num, ep, 1, 0, NULL);
+		writearticles(num, num + 1);
+	    }
 	    else if (isfile(filename)) {
 		unlink(filename);
 	    }
@@ -1148,6 +1227,7 @@ void update_deletions(int num_old)
     }
     set_overwrite = save_ov;
 }
+
 /*
 ** Printing...the other main part of this program!
 ** This writes out the articles, beginning with the number startnum.
@@ -1239,7 +1319,7 @@ void writearticles(int startnum, int maxnum)
 	    free(filename);
 	    continue;
 	}
-	else if (!newfile && !set_overwrite
+	else if (!newfile && !set_overwrite && !has_new_replies(email)
 		 && !(email->is_deleted && set_delete_msgnum)) {
 	    skip = 1;		/* is this really necessary with continue ??? */
 	    num++;
@@ -1263,7 +1343,7 @@ void writearticles(int startnum, int maxnum)
 	print_msg_header(fp, set_label,
 			 email->subject, set_dir,
 			 email->name, email->emailaddr, email->msgid,
-			 email->charset, filename);
+			 email->charset, email->date, filename);
 
 	printcomment(fp, "received", email->fromdatestr);
 	printcomment(fp, "isoreceived", secs_to_iso(email->fromdate));
@@ -1421,6 +1501,7 @@ void writearticles(int startnum, int maxnum)
 			ptr = convchars(email_next_in_thread->subject));
 		if (ptr)
 		    free(ptr);
+		email->initial_next_in_thread = email_next_in_thread->msgnum;
 	    }
 
 	    /*
@@ -2352,6 +2433,107 @@ void writeauthors(int amountmsgs, struct emailinfo *email)
 	putchar('\n');
 }
 
+/*
+** Pretty-prints the items for the haof
+*/
+void printhaofitems(FILE *fp, struct header *hp, int year, int month,
+	       struct emailinfo *subdir_email)
+{
+  char *subj, *from_name, *from_emailaddr;
+
+  if (hp != NULL) {
+    struct emailinfo *em=hp->data;
+    printhaofitems(fp, hp->left, year, month, subdir_email);
+    if ((year == -1 || year_of_datenum(em->date) == year)
+	&& (month == -1 || month_of_datenum(em->date) == month)
+	&& !em->is_deleted
+	&& (!subdir_email || subdir_email->subdir == em->subdir)) {
+
+      subj = convchars(em->subject);
+      from_name = convchars(em->name);
+      from_emailaddr = convchars(em->emailaddr);
+
+      fprintf(fp,
+         "      <mail>\n"
+         "        <subject>%s</subject>\n"
+         "        <date>%s</date>\n"
+         "        <fromname>%s</fromname>\n"
+         "        <fromemail>%s</fromemail>\n" 
+         "        <message-id>%s</message-id>\n"
+         "        <file>\"%s\"</file>\n"
+         "      </mail>\n\n"
+         ,subj, getdatestr(em->date), from_name, from_emailaddr, em->msgid,
+         msg_relpath(em,subdir_email));
+
+      free(subj);
+      free(from_name);
+      free(from_emailaddr);
+    }
+    printhaofitems(fp, hp->right, year, month, subdir_email);
+  }
+}
+
+/*
+** Write the XML based hypermail archive overview file
+*/
+
+void writehaof(int amountmsgs, struct emailinfo *email)
+{
+    int newfile;
+    char *filename;
+    FILE *fp;
+    char *authname = index_name[email && email->subdir != NULL][AUTHOR_INDEX];
+
+    filename = haofname(email);
+    printf("haofname %s\n", filename);
+
+    if (isfile(filename))
+	newfile = 0;
+    else
+	newfile = 1;
+
+    if ((fp = fopen(filename, "w")) == NULL) {
+	sprintf(errmsg, "%s \"%s\".", lang[MSG_COULD_NOT_WRITE], filename);
+	progerr(errmsg);
+    }
+
+    if (set_showprogress)
+	printf("%s \"%s\"...", lang[MSG_WRITING_HAOF], filename);
+
+    fprintf(fp,"<?xml version=\"1.0\" encoding=\"ISO-8859-15\"?>\n\n");
+    fprintf(fp,"  <!DOCTYPE haof PUBLIC "
+        "\"-//Bernhard Reiter//DTD HOAF 0.2//EN\"\n" 
+        "\"http://ffii.org/~breiter/probe/haof-0.2.dtd\">\n\n"
+          );
+    fprintf(fp,"  <haof version=\"0.2\">\n\n");
+    fprintf(fp,"      <archiver name=\"hypermail\" version=\"" 
+		    		VERSION ".pl" PATCHLEVEL "\" />\n\n");
+
+    print_haof_indices(fp, email ? email->subdir : NULL);
+
+	    
+    fprintf(fp,"  <mails>\n");
+    printhaofitems(fp, datelist, -1, -1, email);
+    fprintf(fp,"  </mails>\n");
+
+    fprintf(fp,"  </haof>\n");
+
+    fclose(fp);
+
+    if (newfile && chmod(filename, set_filemode) == -1) {
+	sprintf(errmsg, "%s \"%s\": %o.",
+		lang[MSG_CANNOT_CHMOD], filename, set_filemode);
+	progerr(errmsg);
+    }
+    free(filename);
+
+    if (set_showprogress)
+	putchar('\n');
+}
+
+
+
+
 static int count_messages(struct header *hp, int year, int mo,
 			  long *first_date, long *last_date)
 {
@@ -2611,6 +2793,8 @@ void write_toplevel_indices(int amountmsgs)
 		    writeattachments(sd->count, sd->first_email);
 		    break;
 	    }
+	if (set_writehaof)
+			writehaof(sd->count, sd->first_email);
 
 	    if (!fp) continue;
 	    if (!sd->count) {
