@@ -1,5 +1,5 @@
 /*
-** $Id: setup.c,v 1.26 2001/11/22 00:39:37 pcm Exp $
+** $Id: setup.c,v 1.30 2002/07/23 01:07:26 pcm Exp $
 */
 
 #include "hypermail.h"
@@ -47,10 +47,10 @@ bool set_linkquotes;
 bool set_monthly_index;
 bool set_yearly_index;
 bool set_spamprotect;
+bool set_spamprotect_id;
 bool set_attachmentsindex;
-#ifdef GDBM
 bool set_usegdbm;
-#endif
+bool set_writehaof;
 bool set_append;
 char *set_append_filename;
 bool set_warn_surpressions;
@@ -109,6 +109,7 @@ char *set_unsafe_chars;
 
 char *set_folder_by_date;
 char *set_latest_folder;
+char *set_base_url;
 char *set_describe_folder;
 int set_msgsperfolder;
 
@@ -299,12 +300,18 @@ struct Config cfg[] = {
     {"uselock", &set_uselock, BTRUE, CFG_SWITCH,
      "# Set this to On to use hypermail's internal locking mechanism.\n"},
 
-#ifdef GDBM
     {"usegdbm",  &set_usegdbm,  BFALSE,    CFG_SWITCH,
      "# Set this to On to use gdbm to implement a header cache.\n"
      "#This will speed up hypermail, especially if your filesystem is slow.\n"
-     "#It will not provide any speedup with the linkquotes option.\n"},
+     "#It will not provide any speedup with the linkquotes option.\n"
+#ifndef GDBM
+     "#(This particular binary has been build with GDBM disabled.)\n"
 #endif
+    },
+
+    {"writehaof", &set_writehaof, BFALSE, CFG_SWITCH,
+     "# Set this to On to let hypermail write an XML archive overview file\n"
+     "# in each directory. The filename is " HAOF_NAME ".\n"},
 
     {"append",  &set_append,  BFALSE,    CFG_SWITCH,
      "# Set this to On to maintain a parallel mbox archive. The file\n"
@@ -435,6 +442,12 @@ struct Config cfg[] = {
      "# Set this to On to make hypermail not output real email addresses\n"
      "# in the output HTML but instead it will obfuscate them a little.\n"},
 
+    {"spamprotect_id", &set_spamprotect_id, BFALSE, CFG_SWITCH,
+     "# Set this to On to make hypermail not output real email message\n"
+     "# ids in HTML comments (sometimes used internally by hypermail) but\n"
+     "# instead it will obfuscate them a little so they don't look like\n"
+     "# email addresses to spammers.\n"},
+
     {"attachmentsindex", &set_attachmentsindex, BTRUE, CFG_SWITCH,
      "# Set this to  Off to make hypermail not output an index of\n"
      "# messages with attachments.\n"},
@@ -529,6 +542,11 @@ struct Config cfg[] = {
      "# subdirectory. Note that many web servers are configured to\n"
      "# not follow symbolic links for security reasons.\n"},
 
+    {"base_url", &set_base_url, NULL, CFG_STRING,
+     "# The url of the archive's main directory. This is needed when\n"
+     "# the latest_folder option is used and the folder_by_date makes\n"
+     "# directories more than one level deep (e.g. with '%y/%m').\n"},
+
     {"iso2022jp", &set_iso2022jp, BFALSE, CFG_SWITCH,
      "# Set this to On to support ISO-2022-JP messages.\n"},
 
@@ -1032,9 +1050,8 @@ void dump_config(void)
     printf("set_uselock = %d\n",set_uselock);
     printf("set_locktime = %d\n",set_locktime);
     printf("set_ietf_mbox = %d\n",set_ietf_mbox);
-#ifdef GDBM
     printf("set_usegdbm = %d\n",set_usegdbm);
-#endif
+    printf("set_writehaof = %d\n",set_writehaof);
     printf("set_append = %d\n",set_append);
     printf("set_thrdlevels = %d\n",set_thrdlevels);
     printf("set_dirmode = %04o\n",set_dirmode);
