@@ -618,6 +618,22 @@ static void translatechars(char *start, char *end, struct Push *buff)
     }
 }
 
+static char *translateurl(char *url)
+{
+    char *p;
+    struct Push buff;
+    int in_ascii = TRUE, esclen = 0;
+    INIT_PUSH(buff);
+
+    for (p = url; *p; p++) {
+	if (*p == '&')
+	    PushString(&buff, "&amp;");
+	else
+	    PushByte(&buff, *p);
+    }
+    RETURN_PUSH(buff);
+}
+
 /* Given a string, replaces all instances of "oldpiece" with "newpiece".
 **
 ** Modified this routine to eliminate recursion and to avoid infinite
@@ -752,6 +768,22 @@ char *makemailcommand(char *mailcommand, char *email, char *id, char *subject)
     return newcmd;
 }
 
+char *unspamify(char *s)
+{
+    char *p;
+    if (!s)
+	return s;
+    if (!strchr(s, '@') && ((p = strstr(s, "_at_")) != NULL)) {
+	struct Push buff;
+	INIT_PUSH(buff);
+	PushNString(&buff, s, p - s);
+	PushByte(&buff, '@');
+	PushString(&buff, p + 4);
+	return PUSH_STRING(buff);
+    }
+    return strsav(s);
+}
+
 /*
 ** RFC 1738
 ** Thus, only alphanumerics, the special characters "$-_.+!*'(),", and
@@ -820,7 +852,11 @@ char *parseemail(char *input,	/* string to parse */
 		else
 		    break;
 	    }
-	    if (email != ptr - 1) {	/* bigger chance this is an address */
+	    if (backoff > 2 && email[0] == '/' && email[-1] == '/'
+		&& email[-2] == ':') {
+	        ;		/* part of a url such as ftp://user@host.com */
+	    }
+	    else if (email != ptr - 1) { /* bigger chance this is an address */
 		email++;
 		if (sscanf
 		    (ptr + 1, "%255[" VALID_IN_EMAIL_DOMAINNAME "]",
@@ -1022,11 +1058,13 @@ char *parseurl(char *input)
 		}
 	    }
 	    if(accepted) {
-	        trio_snprintf(tempbuff, sizeof(tempbuff),
+	        char *urlbuff2 = translateurl(urlbuff);
+		trio_snprintf(tempbuff, sizeof(tempbuff),
 			      "<a href=\"%s%s\">%s%s</a>",
-			      thisprotocol, urlbuff, thisprotocol, urlbuff);
+			      thisprotocol, urlbuff2, thisprotocol, urlbuff2);
 		PushString(&buff, tempbuff); /* append the tag buffer */
-		inputp += strlen(urlbuff);
+		inputp += strlen(urlbuff2);
+		free(urlbuff2);
 	    } else {
 	        PushString(&buff, thisprotocol);
 	    }
